@@ -190,13 +190,13 @@ def write_boolean(file: BinaryIO, booleans: List[bool], all_defined: bool = Fals
 
 def read_utf16(file: BinaryIO) -> str:
     """read a utf-16 string from file"""
-    val = ""
+    val = b""
     for _ in range(MAX_LENGTH):
         ch = file.read(2)
         if ch == unhexlify("0000"):
             break
-        val += ch.decode("utf-16LE")
-    return val
+        val += ch
+    return val.decode("utf-16LE")
 
 
 def write_utf16(file: BinaryIO, val: str):
@@ -224,12 +224,12 @@ class PackInfo:
     ]
 
     def __init__(self) -> None:
-        self.packpos = 0  # type: int
-        self.numstreams = 0  # type: int
-        self.packsizes = []  # type: List[int]
-        self.digestdefined = []  # type: List[bool]
-        self.crcs = []  # type: List[int]
-        self.enable_digests = True
+        self.packpos: int = 0
+        self.numstreams: int = 0
+        self.packsizes: List[int] = []
+        self.digestdefined: List[bool] = []
+        self.crcs: List[int] = []
+        self.enable_digests: bool = True
 
     @classmethod
     def retrieve(cls, file: BinaryIO):
@@ -311,21 +311,21 @@ class Folder:
     ]
 
     def __init__(self) -> None:
-        self.unpacksizes = []  # type: List[int]
-        self.coders = []  # type: List[Dict[str, Any]]
-        self.bindpairs = []  # type: List[Bond]
-        self.packed_indices = []  # type: List[int]
+        self.unpacksizes: List[int] = []
+        self.coders: List[Dict[str, Any]] = []
+        self.bindpairs: List[Bond] = []
+        self.packed_indices: List[int] = []
         # calculated values
         # internal values
-        self.solid = False  # type: bool
-        self.digestdefined = False  # type: bool
-        self.crc = None  # type: Optional[int]
+        self.solid: bool = False
+        self.digestdefined: bool = False
+        self.crc: Optional[int] = None
         # compress/decompress objects
-        self.decompressor = None  # type: Optional[SevenZipDecompressor]
-        self.compressor = None  # type: Optional[SevenZipCompressor]
+        self.decompressor: Optional[SevenZipDecompressor] = None
+        self.compressor: Optional[SevenZipCompressor] = None
         self.files = None
         # encryption
-        self.password = None  # type: Optional[str]
+        self.password: Optional[str] = None
 
     @classmethod
     def retrieve(cls, file: BinaryIO):
@@ -343,7 +343,7 @@ class Folder:
             iscomplex = b & 0x10 == 0x10
             hasattributes = b & 0x20 == 0x20
             if methodsize > 0:
-                c = {"method": file.read(methodsize)}  # type: Dict[str, Any]
+                c: Dict[str, Any] = {"method": file.read(methodsize)}
             else:
                 c = {"method": b"\x00"}
             if iscomplex:
@@ -461,7 +461,7 @@ class UnpackInfo:
         return obj
 
     def __init__(self):
-        self.numfolders = None
+        self.numfolders: int = 0
         self.folders = []
         self.datastreamidx = None
 
@@ -536,10 +536,10 @@ class SubstreamsInfo:
     ]
 
     def __init__(self):
-        self.digests = []  # type: List[int]
-        self.digestsdefined = []  # type: List[bool]
-        self.unpacksizes = None  # type: Optional[List[int]]
-        self.num_unpackstreams_folders = []  # type: List[int]
+        self.digests: List[int] = []
+        self.digestsdefined: List[bool] = []
+        self.unpacksizes: Optional[List[int]] = None
+        self.num_unpackstreams_folders: List[int] = []
 
     @classmethod
     def retrieve(cls, file: BinaryIO, numfolders: int, folders: List[Folder]):
@@ -625,9 +625,9 @@ class StreamsInfo:
     __slots__ = ["packinfo", "unpackinfo", "substreamsinfo"]
 
     def __init__(self):
-        self.packinfo = None  # type: PackInfo
-        self.unpackinfo = None  # type: UnpackInfo
-        self.substreamsinfo = None  # type: Optional[SubstreamsInfo]
+        self.packinfo: Optional[PackInfo] = None
+        self.unpackinfo: Optional[UnpackInfo] = None
+        self.substreamsinfo: Optional[SubstreamsInfo] = None
 
     @classmethod
     def retrieve(cls, file: BinaryIO):
@@ -644,6 +644,8 @@ class StreamsInfo:
             self.unpackinfo = UnpackInfo.retrieve(file)
             pid = file.read(1)
         if pid == PROPERTY.SUBSTREAMS_INFO:
+            if self.unpackinfo is None:
+                raise Bad7zFile("Header is broken")
             self.substreamsinfo = SubstreamsInfo.retrieve(file, self.unpackinfo.numfolders, self.unpackinfo.folders)
             pid = file.read(1)
         if pid != PROPERTY.END:
@@ -661,10 +663,14 @@ class StreamsInfo:
 
 
 class HeaderStreamsInfo(StreamsInfo):
+    """
+    Header version of StreamsInfo
+    """
+
     def __init__(self):
         super().__init__()
-        self.packinfo = PackInfo()
-        self.unpackinfo = UnpackInfo()
+        self.packinfo: PackInfo = PackInfo()
+        self.unpackinfo: UnpackInfo = UnpackInfo()
         self.unpackinfo.numfolders = 1
 
     def write(self, file: BinaryIO):
@@ -680,8 +686,8 @@ class FilesInfo:
     __slots__ = ["files", "emptyfiles", "antifiles"]
 
     def __init__(self):
-        self.files = []  # type: List[Dict[str, Any]]
-        self.emptyfiles = []  # type: List[bool]
+        self.files: List[Dict[str, Any]] = []
+        self.emptyfiles: List[bool] = []
         self.antifiles = None
 
     @classmethod
@@ -706,7 +712,7 @@ class FilesInfo:
             buffer = io.BytesIO(fp.read(size))
             if prop == PROPERTY.EMPTY_STREAM:
                 isempty = read_boolean(buffer, numfiles, checkall=False)
-                list(map(lambda x, y: x.update({"emptystream": y}), self.files, isempty))  # type: ignore
+                list(map(lambda x, y: x.update({"emptystream": y}), self.files, isempty))
                 numemptystreams += isempty.count(True)
             elif prop == PROPERTY.EMPTY_FILE:
                 self.emptyfiles = read_boolean(buffer, numemptystreams, checkall=False)
@@ -889,15 +895,16 @@ class WriteWithCrc:
 class Header:
     """the archive header"""
 
-    __slot__ = ["solid", "main_streams", "files_info", "size", "_start_pos"]
+    __slot__ = ["solid", "main_streams", "files_info", "size", "_start_pos", "_initialized", "filters"]
 
     def __init__(self) -> None:
-        self.solid = False
+        self.solid: bool = False
         self.main_streams = None
         self.files_info = None
-        self.size = 0
-        self._start_pos = 0
-        self.password = None  # type: Optional[str]
+        self.size: int = 0
+        self._start_pos: int = 0
+        self.password: Optional[str] = None
+        self._initialized: bool = False
 
     @classmethod
     def retrieve(cls, fp: BinaryIO, buffer: BytesIO, start_pos: int, password=None):
@@ -1016,21 +1023,49 @@ class Header:
             raise Bad7zFile("end id expected but %s found" % (repr(pid)))  # pragma: no-cover
 
     @staticmethod
-    def build_header(folders):
+    def build_header(filters, password):
         header = Header()
-        header.files_info = FilesInfo()
-        header.main_streams = StreamsInfo()
-        header.main_streams.packinfo = PackInfo()
-        header.main_streams.packinfo.numstreams = 0
-        header.main_streams.packinfo.packpos = 0
-        header.main_streams.unpackinfo = UnpackInfo()
-        header.main_streams.unpackinfo.numfolders = len(folders)
-        header.main_streams.unpackinfo.folders = folders
-        header.main_streams.substreamsinfo = SubstreamsInfo()
-        header.main_streams.substreamsinfo.num_unpackstreams_folders = [len(folders)]
-        header.main_streams.substreamsinfo.unpacksizes = []
+        header.password = password
+        header.filters = filters
         return header
 
+    def initialize(self):
+        if not self._initialized:
+            self._initialized = True
+            if self.main_streams is not None:
+                # append mode
+                folder = Folder()
+                folder.password = self.password
+                folder.prepare_coderinfo(self.filters)
+                self.main_streams.packinfo.enable_digests = False  # FIXME
+                self.main_streams.unpackinfo.folders.append(folder)
+                self.main_streams.unpackinfo.numfolders += 1
+                self.main_streams.substreamsinfo.num_unpackstreams_folders.append(0)
+            else:
+                # create new header
+                folder = Folder()
+                folder.password = self.password
+                folder.prepare_coderinfo(self.filters)
+                folders = [folder]
+                self.files_info = FilesInfo()
+                self.main_streams = StreamsInfo()
+                self.main_streams.packinfo = PackInfo()
+                self.main_streams.packinfo.numstreams = 0
+                self.main_streams.packinfo.packpos = 0
+                self.main_streams.unpackinfo = UnpackInfo()
+                self.main_streams.unpackinfo.numfolders = len(folders)
+                self.main_streams.unpackinfo.folders = folders
+                self.main_streams.substreamsinfo = SubstreamsInfo()
+                self.main_streams.substreamsinfo.num_unpackstreams_folders = [len(folders)]
+                self.main_streams.substreamsinfo.unpacksizes = []
+                self.main_streams.packinfo.enable_digests = self.password is not None
+                self.main_streams.packinfo.numstreams = 0
+                self.main_streams.substreamsinfo.digests = []
+                self.main_streams.substreamsinfo.digestsdefined = []
+                self.main_streams.substreamsinfo.num_unpackstreams_folders = [0]
+                self.main_streams.packinfo.packsizes = []
+                self.main_streams.packinfo.crcs = []
+
 
 class SignatureHeader:
     """The SignatureHeader class hold information of a signature header of archive."""
@@ -1048,10 +1083,10 @@ class SignatureHeader:
             P7ZIP_MAJOR_VERSION,
             P7ZIP_MINOR_VERSION,
         )  # type: Tuple[bytes, ...]
-        self.startheadercrc = -1  # type: int
-        self.nextheaderofs = -1  # type: int
-        self.nextheadersize = -1  # type: int
-        self.nextheadercrc = -1  # type: int
+        self.startheadercrc: int = -1
+        self.nextheaderofs: int = -1
+        self.nextheadersize: int = -1
+        self.nextheadercrc: int = -1
 
     @classmethod
     def retrieve(cls, file: BinaryIO):
