@@ -311,10 +311,8 @@ class PpmdDecompressor(ISevenZipDecompressor):
         self.decoder = pyppmd.Ppmd7Decoder(order, mem)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length=-1) -> bytes:
-        if max_length <= 0:
-            return self.decoder.decode(data, 1)
-        if len(data) == 0:
-            return self.decoder.flush(max_length)
+        if len(data) == 0 and self.decoder.needs_input:
+            return self.decoder.decode(b"\0", max_length)
         return self.decoder.decode(data, max_length)
 
 
@@ -630,10 +628,7 @@ class SevenZipDecompressor:
     def _decompress(self, data, max_length: int):
         for i, decompressor in enumerate(self.chain):
             if self._unpacked[i] < self._unpacksizes[i]:
-                if isinstance(decompressor, LZMA1Decompressor) or isinstance(decompressor, PpmdDecompressor):
-                    data = decompressor.decompress(data, max_length)  # always give max_length for lzma1
-                else:
-                    data = decompressor.decompress(data)
+                data = decompressor.decompress(data, max_length)
                 self._unpacked[i] += len(data)
             elif len(data) == 0:
                 data = b""
